from netq_lib.common.netq_pq import NetQPriorityQueue, QueueElem
from collections import defaultdict
import enum


class FullState(object):

    def __init__(self, msg_type):
        self.decorated = self
        self.msg_type = msg_type

    def __call__(self, func):
        def wrapper(obj):
            if obj.full_state_object:
                obj.full_state_object.set_fullstate(True)
                obj.full_state_object.empty_pq(self.msg_type)
            func(obj)
            if obj.full_state_object:
                obj.full_state_object.set_fullstate(False)
                obj.full_state_object.flush_fullstate(self.msg_type)

        return wrapper

class FullStateMessages(enum.Enum):
    FULL_STATE = 0
    PARTIAL_STATE = 1
    FULL_STATE_SEGEMENT = 2
    FULL_STATE_COMPLETE = 3


class NetLinkFullStateObject(object):

    def __init__(self):
        self.__netlink_event = defaultdict(dict)
        self.__netlink_full_state = False
        self.pq = NetQPriorityQueue.PQ

    def set_fullstate(self, state):
        self.__netlink_full_state = state

    def get_fullstate(self):
        return self.__netlink_full_state

    def cache_netlink_buffer(self, obj, ifindex):
        self.__netlink_event[obj.message_type][ifindex] = obj

    def empty_pq(self, message_type):
        pq = NetQPriorityQueue.PQ

        priority = pq.get_pqid_by_message(message_type)
    
        pq.empty_pq(priority)


    def flush_fullstate(self, message_type):
        message = {}
        pq = NetQPriorityQueue.PQ

        message['message_type'] = message_type
        message['priority_qid'] = pq.get_pqid_by_message(message_type)
        priority = self.__get_message_type_priority(message_type)

        try:
            queue_elem = QueueElem(priority, "FULL_STATE_END", True, message)
            pq.putq(queue_elem)
        except Exception as e:
            return False

        #* MACFDB generates VxLanRemoteDes */
        if(message_type == "MacFdb"):
            self.flush_fullstate("VxlanRemoteDest")


    def __get_message_type_priority(self, message_type):
        return 1

    def __reset_event_buff(self):
        self.__netlink_event = defaultdict(dict)

    def flush_netlink_buffer(self):

        pq = NetQPriorityQueue.PQ
        if pq is None:
            return False

        for message_type, interfaces in self.__netlink_event.iteritems():
            out_msg = ""

            priority = self.__get_message_type_priority(message_type)

            message_data = [message.to_object_json() for ifidx, message in interfaces.iteritems()]

            try:
                queue_elem = QueueElem(priority, message_type, True, message_data)
                pq.putq(queue_elem)
            except Exception as e:
                return False

        self.__reset_event_buff()



NetlinkFullStateObject = NetLinkFullStateObject()
